//
// Created by 乾三 on 2023/2/23.
//

#include "../include/onlyDetectInference.h"
#include <iostream>
#include <sys/timeb.h>
#include <cmath>
#include <algorithm>
#include "acl/acl.h"
#include "AclLiteApp.h"
#include "AclLiteModel.h"
#include "CarParams.h"
//#include "inference.h"

using namespace std;

// 这里只是模型和推理函数
helmetDetectInferenceThread::helmetDetectInferenceThread(aclrtRunMode& runMode,const int type)
        :runMode_(runMode), imageInfoSize_(0),imageInfoBuf_(nullptr), classifyInputSize_(0), classifyInputBuf_(nullptr)
{
    ModelSwith(type);
}

helmetDetectInferenceThread::~helmetDetectInferenceThread()
{
//    detectModel_.DestroyResource();
//    classifyModel_.DestroyResource();
}



AclLiteError helmetDetectInferenceThread::Init()
{

    AclLiteError ret = detectModel_.Init(kDetectModelPath_);
    if (ret != ACLLITE_OK) {
        ACLLITE_LOG_ERROR("detect Model init failed, error:%d", ret);
        return ret;
    }

    return ACLLITE_OK;
}

AclLiteError helmetDetectInferenceThread::DetectModelExecute(shared_ptr<ObjDetectDataMsg> objDetectDataMsg)
{
    if (objDetectDataMsg->isLastFrame == 1) {
        ACLLITE_LOG_INFO("it is channel %d's lastframe in Detect Inference", objDetectDataMsg->channelId);
        return ACLLITE_OK;
    }
    AclLiteError ret = detectModel_.CreateInput(objDetectDataMsg->resizedFrame.data.get(),
                                                objDetectDataMsg->resizedFrame.size);
    if (ret != ACLLITE_OK) {
        ACLLITE_LOG_ERROR("Create detect model input dataset failed");
        return ACLLITE_ERROR;
    }

    ret = detectModel_.Execute(objDetectDataMsg->detectInferData);
    if (ret != ACLLITE_OK) {
        ACLLITE_LOG_ERROR("Execute detect model inference failed, error: %d", ret);
        return ACLLITE_ERROR;
    }
    detectModel_.DestroyInput();
    aclrtFree(imageInfoBuf_);
    imageInfoBuf_ = nullptr;
    return ACLLITE_OK;
}



AclLiteError helmetDetectInferenceThread::Process(int msgId, shared_ptr<void> data)
{
    switch (msgId) {
        case MSG_DETECT_PREPROC_DATA:
            DetectModelExecute(static_pointer_cast<ObjDetectDataMsg>(data));
            DetectMsgSend(static_pointer_cast<ObjDetectDataMsg>(data));
            break;
        default:
            ACLLITE_LOG_INFO("Inference thread ignore msg %d", msgId);
            break;
    }

    return ACLLITE_OK;
}

AclLiteError helmetDetectInferenceThread::DetectMsgSend(shared_ptr<ObjDetectDataMsg> objDetectDataMsg)
{
    while (1) {
        AclLiteError ret = SendMessage(objDetectDataMsg->detectPostThreadId, MSG_DETECT_INFER_OUTPUT, objDetectDataMsg);
        if (ret == ACLLITE_ERROR_ENQUEUE) {
            usleep(500);
            continue;
        } else if(ret == ACLLITE_OK) {
            break;
        } else {
            ACLLITE_LOG_ERROR("Send read frame message failed, error %d", ret);
            return ret;
        }
    }

    return ACLLITE_OK;
}

// 工服  --------------------------------------------------------------




AclLiteError helmetDetectInferenceThread::ModelSwith(int type) {
    switch (type) {
        case INSTANCENAME::CAR_COLOR_INFERENCE_INSTANCE : kDetectModelPath_ = "../model/yolov3.om"; break;
        case INSTANCENAME::HELMET_INFERENCE_INSTANCE : kDetectModelPath_ = "../model/helmet_3class_640x448_yuv420_zipp.om"; break;
        case INSTANCENAME::GONGFU_INFERENCE_INSTANCE : kDetectModelPath_ =  "../weights/yolov5-6.0-gongfu-i14k-448_640_noaipp.om"; break;

        case INSTANCENAME::FIRESMOKE_INFERENCE_INSTANCE : kDetectModelPath_ = "../model/firesmokem_ap95_079_640x448_yuvaipp448.om"; break;
        case INSTANCENAME::YANGCHEN_INFERENCE_INSTANCE : kDetectModelPath_ = "../model/yangchen/yangchen_yolo5s_640_448_ap95_044_rgb.om"; break;
        case INSTANCENAME::COCO_PERSON_INFERENCE_INSTANCE : kDetectModelPath_ = "../model/coco/yolo5s_coco_640x448_yuvaipp448_rgb.om"; break;
        default : break;
    }
    return ACLLITE_OK;
}

